const express = require('express');
const path = require('path');
const bodyParser = require('body-parser');
const logger = require('morgan');
const http = require('http');
const OpenApiValidator = require('express-openapi-validator');

const port = 3000;
const app = express();
const apiSpec = path.join(__dirname, 'api.yaml');

app.use(bodyParser.urlencoded({ extended: false }));
app.use(bodyParser.text());
app.use(bodyParser.json());

app.use(logger('dev'));
app.use('/spec', express.static(apiSpec));

// 1. Create a router
const api = express.Router({ mergeParams: true });

// 2. Use oav middleware with router
api.use(
  OpenApiValidator.middleware({
    apiSpec,
    validateResponses: true,
    operationHandlers: path.join(__dirname),
  })
);

// use the router on the app
app.use(api);

// actually any handler after API handler
app.use((req, res, next) => {
  res.status(404).send({ error: 'Page not found.' });
});

app.use((err, req, res, next) => {
  // format errors
  console.error(err);
  res.status(err.status || 500).json({
    message: err.message,
    errors: err.errors,
  });
});

http.createServer(app).listen(port);
console.log(`Listening on port ${port}`);

module.exports = app;
